/*
 * Stonage: a puzzle game
 * Copyright (C) 2001 Luc Saillard <luc@saillard.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
/*
 * Source for loading/drawing square
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif
#include "square.h"
#include "image.h"
#include "trace.h"
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtkwidget.h>

/*
 * Extract the basename on the full filename
 */
const char *basename(const char *filename)
{
  const char *p;

  /* FIXME: change / by \\ on a msdos filesystem */
  p=strrchr(filename,'/');
  if (p==NULL)
   return filename;
  p++;
  return p;
}

int Square_Count_Dir(const char *directory)
{
  int n;
  DIR *d;
  struct dirent *dir;
#if ENABLE_DEBUG_TRACE
  char path[1024];
#endif

  n=0;
  d=opendir(directory);
  if (d==NULL)
   {
     trace("Can't open directory %s (%s)",directory,strerror(errno));
     return 0;
   }

  while ((dir=readdir(d))!=NULL)
   {
     if (dir->d_name[0]=='.')
       continue;
#ifdef ENABLE_DEBUG_TRACE
     snprintf(path,1023,"%s/%s",directory,dir->d_name);
     trace("%s",path);
#endif
     n++;
   }

  closedir(d);
  return n;
}


int Square_Load_Dir(GList **squares,const char *directory)
{
  int n;
  DIR *d;
  struct dirent *dir;
  char path[1024];
  GtkWidget *pixmap;
  struct t_square *square;

  n=0;
  d=opendir(directory);
  if (d==NULL)
   {
     trace("Can't open directory %s (%s)",directory,strerror(errno));
     return 0;
   }

  while ((dir=readdir(d))!=NULL)
   {
     if (*(dir->d_name)=='.')
       continue;
     snprintf(path,1023,"%s/%s",directory,dir->d_name);
     trace("Trying to load %s",path);
     pixmap = gtk_pixmap_new_from_file(path);
     if (pixmap)
      {
	square = malloc(sizeof(struct t_square));
	if (square==NULL)
	 {
	   trace("Failed to alloc %d bytes",sizeof(struct t_square));
	   return 0;
	 }
	trace("Loading %s: success",path);
	memset(square,0,sizeof(struct t_square));
	square->pixmap = pixmap;
	*squares = g_list_append(*squares,square);
	n++;
      }
     else
      {
	trace("Loading %s: failed",path);
      }
   }

  closedir(d);
  return n;
}

/*
 * Count the number of graphics squares available
 */
int Square_Count(void)
{
  int i,n;
  char tmp[1024];
  char *datadir_path[]=
   {
     STONAGE_DATA_DIR,
     ".",
     "../data",
     NULL
   };
  i=0; n=0;
  while (datadir_path[i])
   {
     snprintf(tmp,1024,"%s/squares/",datadir_path[i]);
     trace("Trying to parse %s",tmp);
     n += Square_Count_Dir(tmp);
     i++;
   }
  return n;
}

/*
 * Load all the square
 */
int Square_Load_All(GList **squares)
{
  int i,n;
  char tmp[1024];
  char *datadir_path[]=
   {
     STONAGE_DATA_DIR,
     ".",
     "../data",
     NULL
   };
  i=0; n=0;
  while (datadir_path[i])
   {
     snprintf(tmp,1024,"%s/squares/",datadir_path[i]);
     trace("Trying to parse %s",tmp);
     n += Square_Load_Dir(squares,tmp);
     i++;
   }
  return n;
}




