/*
 * Stonage: a puzzle game
 * Copyright (C) 2001 Luc Saillard <luc@saillard.org>
 * 
 * Source for loading/drawing images
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "stonage.h"
#include "image.h"
#include "trace.h"
#include <stdlib.h>
#include <stdio.h>
#include <SDL.h>
#if defined(HAVE_SDL_SDL_IMAGE_H)
#include <SDL/SDL_image.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

static char *datadir_path[]=
{
  STONAGE_DATA_DIR,
  ".",
  "../data",
  NULL
};


SDL_Surface *
Image_Load(const char *filename, int convert2screenformat)
{
  int i;
  char tmp[1024];
  SDL_Surface *image, *image2;
  i=0; image=NULL;
  while (datadir_path[i])
   {
     snprintf(tmp,1023,"%s/%s",datadir_path[i],filename);
     trace("Trying to load %s",tmp);
#if defined(HAVE_SDL_SDL_IMAGE_H)
     image = IMG_Load(tmp);
#else
     image = SDL_LoadBMP(tmp);
#endif
     if (image)
       break;
     i++;
   }
  if (image==NULL)
   {
     fprintf(stderr,_("Can't load this image %s\n"),filename);
     return NULL;
   }
  
  if (convert2screenformat)
   {
     image2 = SDL_DisplayFormat(image);
     SDL_FreeSurface(image);
     return image2;
   }
  else
    return image;
}

void
Image_Free(SDL_Surface *img)
{
  if (img!=NULL)
   {
     SDL_FreeSurface(img);
   }
}

/*
 * Description: 
 *   Load a image at random in this directory
 */
SDL_Surface *
Image_Load_Random(const char *directory)
{
  int n,i,j;
  DIR *d;
  struct dirent *dir;
  SDL_Surface *img;
#if ENABLE_DEBUG_TRACE
  char path[1024];
#endif
  char realdirectory[1024];

  img=NULL;
  i=n=0;

  while (datadir_path[i])
   {
     snprintf(realdirectory,1023,"%s/%s",datadir_path[i],directory);
     trace("Trying to open directory %s",realdirectory);

     d=opendir(realdirectory);
     if (d!=NULL)
       break;
     trace(_("Can't open directory %s (%s)"),realdirectory,strerror(errno));
     i++;
   }
  if (datadir_path[i]==NULL)
   {
     trace("Found no valid directory to load background");
     return NULL;
   }

  while ((dir=readdir(d))!=NULL)
   {
     if (dir->d_name[0]=='.')
       continue;
     /* We don't accept Makefile has a file image :) */
     if (strstr(dir->d_name,"Makefile"))
       continue;

#ifdef ENABLE_DEBUG_TRACE
     snprintf(path,1023,"%s/%s",realdirectory,dir->d_name);
     trace("%s",path);
#endif
     n++;
   }

  rewinddir(d);

  j=1 + (int)(n*rand()/(RAND_MAX+1.0));

  trace("Image N%d/%d",j,n);

  while ((dir=readdir(d))!=NULL && j)
   {
     if (dir->d_name[0]=='.')
       continue;
     /* We don't accept Makefile has a file image :) */
     if (strstr(dir->d_name,"Makefile"))
       continue;
#ifdef ENABLE_DEBUG_TRACE
     snprintf(path,1023,"%s/%s",realdirectory,dir->d_name);
     trace("%s",path);
#endif
     j--;
   }
  if (j==0)
   {
     snprintf(path,1023,"%s/%s",realdirectory,dir->d_name);
#if defined(HAVE_SDL_SDL_IMAGE_H)
     img = IMG_Load(path);
#else
     img = SDL_LoadBMP(path);
#endif
   }

  closedir(d);

  return img;
}

