/*
 * Stonage: a puzzle game
 * Copyright (C) 2001 Luc Saillard <luc@saillard.org>
 *
 * Header file for loading, manage Level file
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#ifndef __LEVEL_H
#define __LEVEL_H

#include "square.h"

struct level_desc
{
  unsigned char version[4]  __attribute__ ((packed)); /* LVL?, ? is the version number of the file */
  unsigned char sizex[2]    __attribute__ ((packed)); /* width of the playfield  (u16) */
  unsigned char sizey[2]    __attribute__ ((packed)); /* height of the playfield (u16) */
  unsigned char nsquares[2] __attribute__ ((packed)); /* number of differents squares in this file (u16) */

  unsigned char dummy[22];            /* For futher extension */
};

#ifndef _NO_GFX_

#include <SDL.h>

struct t_level
{
  int sizex, sizey, nsquares;
  struct t_square *squares;
  int *map;
  SDL_Surface *background;
  char *comment;
  enum                         /* This is the state machine ... */
   {
     CONTROL_PLAYER,
     CONTROL_KINESIST,
     CONTROL_SQUARE
   } control;
  int xdino,ydino;		/* Dinosaur position in the playfield */
  int xkinesist, ykinesist;	/* Position of the telekinesist */
  int start_x,start_y;		/* Start square position */
  int finish_x, finish_y;	/* End square position TODO: change this to a list
				 we can have more than one exit */
  int keys;                     /* List of key we have collected */
};

enum movement_type
{
  MOVE_DOWN=0,
  MOVE_UP=1,
  MOVE_RIGHT=2,
  MOVE_LEFT=3
};

struct t_level *Level_Load(const char *filename);
int Level_Sanity_Check(struct t_level *l);
int Level_Draw2Screen(struct t_level *l,SDL_Surface *screen);
int Level_MoveDinoTo(struct t_level *l, enum movement_type direction);
int Level_MoveKinesistTo(struct t_level *l, enum movement_type direction);
int Level_CanMoveSquare(struct t_level *l);
int Level_MoveSquareTo(struct t_level *l, enum movement_type direction);
int Level_IsFinish(struct t_level *l);
int Level_DoPostMoveAction(struct t_level *l);
void Level_Free(struct t_level *l);

#endif


#endif

